package edu.umich.visualsoar.graph;

import edu.umich.visualsoar.parser.*;
import edu.umich.visualsoar.operatorwindow.*;

/**
 * This represents an attribute in Working Memomry
 * @author Brad Jones
 * @version 0.5a Oct 1999
 */

public class NamedEdge extends Edge {
///////////////////////////////////////////////////////////////////
// Data Members
///////////////////////////////////////////////////////////////////
	/**
	 * The name of this named edge
	 */
	protected String name = "";

  /**
    * Comments user attributes to this edge on the datamap
    */
  private String comment = "";

  /**
    * Knowledge of whether generated by DataMap generator.
    * If it is true, will show up as green on the datamap
    * until validated by user.
    */
  private boolean generated;

  /**
    * These are used to hold the information of where in the rules that
    * this NamedEdge was generated during the datamap generation.
    */
  private OperatorNode node = null;
  private int lineNumber = 0;
	
///////////////////////////////////////////////////////////////////
// Constructors
///////////////////////////////////////////////////////////////////
	// Deny Default Construction
	/**
	 * A named edge is logically a connection between two
	 * vertices, from v0 to v1, with a name
	 * @param v0 where the edge starts
	 * @param v1 where the edge ends
	 * @param _name the name that you want to name the string, note
	 * if you change this string you also change the name of the node
	 */
	public NamedEdge(Vertex v0, Vertex v1, String _name) {
		super(v0,v1);
		name = _name;
    generated = false;
	}
	
///////////////////////////////////////////////////////////////////
// Methods
///////////////////////////////////////////////////////////////////
	/**
	 * This method returns the name of the node, note if the return
	 * value of this string is changed so is the name of the node
	 * @return the name of this node
	 */
	public String getName() {
		return name;
	}
	
	/**
	 * This method changes the name of the node from whatever it was
	 * to the parameter passed, note changes to s will change the
	 * name of the node
	 * @param s what you want to rename the node to
	 * @return the name of this node
	 */
	public void rename(String s) {
		name  = s;
	}
	
	/**
	 * This presents a String representation of this class
	 * it is the name of the edge followed by representation
	 * of v1's toString() method
	 * @return a string just described
	 */
	public String toString() {
		return name + v1.toString();
	}
	
	/**
	 * This is the write method that writes out a representation
	 * of this edge to the Writer that can later be read back in
	 * to recreate this edge
	 * @param w the stream to write the representation to
	 * @throws IOException if an error writing to the stream occurs
	 */
	public void write(java.io.Writer w) throws java.io.IOException {
		w.write("" + v0.getValue() + ' ' + name + ' ' + v1.getValue() + '\n');
	}
	
	/**
	 * an edge is equal to another object if and only if
	 * the other object is an edge, not null and the starting
	 * ending vertexs are equal and the names are the same
	 * @param o the object that you want to test equality
	 * @return whether the test was true or false like just described
	 */
	public boolean equals(Object o) {
		if (o instanceof NamedEdge && o != null) {
			NamedEdge anotherEdge = (NamedEdge)o;
			if(v0.getValue() == anotherEdge.V0().getValue() && 
			   v1.getValue() == anotherEdge.V1().getValue() && 
			   name.equals(anotherEdge.getName()))
				return true;
			else
				return false;
		}
		else
			return this == o;
	}
	
	/**
	 * satisfies tests whether or not this edge could be used to
	 * satisfy the constraint passed in, if so, it returns true
	 * else false
	 * @param triple the constraint to test satisfaction to
	 * @return whether or not this NamedEdge can satisfy that constraint
	 */
	public boolean satisfies(Triple triple) {
		if (!TripleUtils.isVariable(triple.getAttribute().getString()))
			if (!triple.getAttribute().getString().equals(name))
				return false;
		if (TripleUtils.isVariable(triple.getValue().getString()))
			return true;
		SoarVertex sv = (SoarVertex)V1();
		return sv.isValid(triple.getValue().getString());
	}



  public boolean hasComment() {
    if (comment.equals(""))
      return false;
    else
      return true;
  }

  public void setComment(String newComment) {
    comment = newComment;
  }

  public String getComment() {
    return comment;
  }

  public boolean isGenerated() {
    return generated;
  }

  /*
   *  Called when the datamap automatically generates the edge on the
   *  datamap.  Displayed as green text on datamap.
   */
  public void setAsGenerated() {
    generated = true;
  }

  /*
   *  When user validates the edge generated by the datamap generator
   *  generated variable goes to false and subsequently the datamap know
   *  displays the edge as normal black color text.
   */
  public void validate() {
    generated = false;
    lineNumber = 0;
    node = null;
  }

  public OperatorNode getNode() {
    return node;
  }

  public void setNode(OperatorNode newNode) {
    node = newNode;
  }

  public int getLine() {
    return lineNumber;
  }

  public void setLineNumber(int number) {
    lineNumber = number;
  }

}
